/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software
 * License version 1.1, a copy of which has been included with this
 * distribution in the LICENSE.APL file.  */

package org.apache.log4j.ext;

/**
 * Title: SnmpTrapSenderFacade<br>
 * Description: A simplified interface to an underlying SNMP API.<p>
 * The intent of this class is both to limit the available interface, and allow
 * the underlying SNMP library to be changed, as desired.<br>
 * @author Mark Masterson (<a href="mailto:m.masterson@computer.org">m.masterson@computer.org</a>)<br>
 * <a href="http://www.m2technologies.net/">http://www.m2technologies.net/</a><br>
 * @version 1.0.1
 * changes ---<br>
 *
 * 2002-10-15: mwm : changed the sysUpTime value to a long, to cope with the SysUpTimeResolver mechanism.<br>
 */

public interface SnmpTrapSenderFacade {

    /**
     * This method is called to set the values of all of the class fields used
     * as parameters to the underlying SNMP API.  This method <b>must</b> be
     * called prior to calling either of the other methods in this class.
     *
     * @param managementHostValue - the numeric IP address of the host
     * to which traps should be sent, e.g. "10.255.255.1".
     * <p>
     * @param managementHostTrapListenPortValue - any valid TCP/IP port
     * number.  This is the port that the host specified in the
     * "managementHostValue" variable will listen on for SNMP traps.
     * <p>
     * @param enterpriseOIDValue - formatted as an OID
     * E.g. "1.3.6.1.2.1.1.2.0" -- this OID would point to the standard
     * sysObjectID of the "systemName" node of the standard "system" MIB.
     * <br>
     * If you want(need) to use custom OIDs (such as ones from the
     * "private.enterprises" node -- "1.3.6.1.4.1.x.x.x..."), you always need
     * to provide the <b>fully qualified</b> OID as the value of this
     * variable.
     * <p>
     * @param localIPAddressValue - an IP address, as a String, in
     * numeric, dotted decimal format.  E.g. "10.255.255.2". This is the IP
     * address of the host that is using this appender to send SNMP traps.
     * This address will be encoded in the SNMP PDU, and used to provide
     * things like the "agent"'s IP address.
     * <p>
     * @param localTrapSendPortValue - the port that should be used on the
     * local host by the appender to send the trap.
     * <p>
     * @param genericTrapTypeValue - generic trap type for this SNMP PDU.
     * One of the following values (from the SNMP standard):<br>
     *        0 -- cold start<br>
     *        1 -- warm start<br>
     *        2 -- link down<br>
     *        3 -- link up<br>
     *        4 -- authentification failure<br>
     *        5 -- EGP neighbor loss<br>
     *        6 -- enterprise specific<br>
     * <p>
     * @param specificTrapTypeValue - any value within the range defined
     * for an INTEGER in the ASN.1/BER notation; i.e. -128 to 127.
     * <p>
     * @param communityStringValue - E.g. "public".
     * <p>
     * @param sysUpTimeValue - this is meant to be the amount of time, in
     * seconds, elapsed since the last re-start or re-initialization of the
     * calling application.  Of course, to set this, your application needs
     * to keep track of the value.
     */
    public void initialize(String managementHostValue,
                           int managementHostTrapListenPortValue,
                           String enterpriseOIDValue,
                           String localIPAddressValue,
                           int localTrapSendPortValue,
                           int genericTrapTypeValue,
                           int specificTrapTypeValue,
                           String communityStringValue,
                           long sysUpTimeValue);

    /**
     * Sends the PDU defined by the variables of the fields of this class.
     * This method should always be called last by an application, after having
     * called the initialization() method once, and the addTrapMessageVariable()
     * method one or more times.
     */
    public void sendTrap();

    /**
     * Adds a new Varbind to the SNMP PDU.  The Varbind is made of the value
     * of the application trap OID (applicationTrapOID) parameter, as the key, and the
     * value paramater to this method as the value.  A PDU has a collection
     * of Varbind variables -- repeated calls to this method will add to that
     * collection successively.
     *
     * @param applicationTrapOIDValue - formatted as an OID
     * E.g. "1.3.6.1.2.1.1.2.0.0.0.0" -- this OID would point to the standard
     * sysObjectID of the "systemName" node of the standard "system" MIB.
     * <br>
     * This is the default value, if none is provided.
     * <br>
     * If you want(need) to use custom OIDs (such as ones from the
     * "private.enterprises" node -- "1.3.6.1.4.1.x.x.x..."), you always need
     * to provide the <b>fully qualified</b> OID as the parameter for this
     * variable.
     * <p>
     * @param value - the text to append to the Varbind that will be added to
     * the SNMP PDU.
     */
    void addTrapMessageVariable(String applicationTrapOIDValue,
                                String value);

}