/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software
 * License version 1.1, a copy of which has been included with this
 * distribution in the LICENSE.APL file.  */
package org.apache.log4j.ext;

import org.apache.log4j.helpers.*;
import ca.wengsoft.snmp.Core.*;

/**
 * Title: WengsoftSNMPTrapSender<br>
 * Description: This class makes use of the Wengsoft SNMP library to implement
 * the underlying SNMP protocol(s).<br>
 *
 * You can get a copy of Wengsoft SNMP <a href="http://www.simpleweb.org/
 * software/select_obj.php?orderBy=package&oldorderBy=package&orderDir=
 * asc&free=free">here</a>.<br>
 *
 * You will need a copy of the library ("snmpv3.jar" at a minimum) to use
 * and/or compile this class.<br>
 * <p>
 * The source code of the Wengsoft SNMP library includes the following comment:<br>
 * <pre>
 * COPYRIGHT (c) 1999 by WENG-SOFT Inc. Brosard QC CA<br>
 * All Rights Reserved.<br>
 *<br>
 * The copyright to the computer program(s) herein is the property of WENG-SOFT<br>
 * Inc. It's a free software, The program(s) may be used and/or copied freely.<br>
 *<br>
 * Contact : wweng@videotron.ca<br>
 * </pre>
 * <p>
 * @author Mark Masterson (<a href="mailto:m.masterson@computer.org">m.masterson@computer.org</a>)<br>
 * <a href="http://www.m2technologies.net/">http://www.m2technologies.net/</a><br>
 * @version 1.0.1<br>
 * 2002-10-03<br>
 * changes ---<br>
 *
 * 2002-10-15: mwm : changed the sysUpTime value to a long, to cope with the SysUpTimeResolver mechanism.<br>
 *
 * 2002-12-10: mwm : minor tweaks and prettying up of code.<br>
 */

public class WengsoftSNMPTrapSender implements SnmpTrapSenderFacade {

    private String managementHost = "127.0.0.1";
    private int managementHostTrapListenPort = 162;
    private String enterpriseOID = "1.3.6.1.2.1.1.2.0";
    private long sysUpTime = 0;
    private boolean isInitialized = false;
    private SnmpMessage snmpMessage;
    private SnmpGetSetPdu trapPdu;

    private static final String SYSTEM_UPTIME_KEY = "1.3.6.1.2.1.1.3";
    private static final String TRAP_OID_KEY = "1.3.6.1.6.3.1.1.4.1";
    private static final String ENTERPRISE_OID_KEY = "1.3.6.1.6.3.1.1.4.3";

    /**
     * Default constructor.
     */
    public WengsoftSNMPTrapSender() {
    }

    public void initialize(final String managementHostValue,
                           final int managementHostTrapListenPortValue,
                           final String enterpriseOIDValue,
                           final String localIPAddressValue,
                           final int localTrapSendPortValue,
                           final int genericTrapTypeValue,
                           final int specificTrapTypeValue,
                           final String communityStringValue,
                           final long sysUpTimeValue) {
        this.managementHost = managementHostValue;
        this.managementHostTrapListenPort = managementHostTrapListenPortValue;
        this.enterpriseOID = enterpriseOIDValue;
        this.sysUpTime = sysUpTimeValue;
        this.snmpMessage = new SnmpMessage();
        this.trapPdu = new SnmpGetSetPdu();
        this.isInitialized = true;
    }

    public void addTrapMessageVariable(final String applicationTrapOIDValue, final String value) {
        //check pre-condition
        if (!this.isInitialized) {
            LogLog.error("The initialize() method must be called before calling addTrapMessageVariable()");
            return;
        }
        this.trapPdu.addNameValuePair(new AsnNameValuePair(TRAP_OID_KEY, new AsnOID(applicationTrapOIDValue)));
        this.trapPdu.addNameValuePair(new AsnNameValuePair(applicationTrapOIDValue, new AsnOctets(value)));
    }

    public void sendTrap() {
        //check pre-condition
        if (!this.isInitialized) {
            LogLog.error("The initialize() method must be called before calling sendTrap()");
            return;
        }
        this.trapPdu.setMsgType(AsnObject.SNMPV2_TRAP);
        this.trapPdu.setReqId(1);
        // Add enterprise OID
        this.trapPdu.addNameValuePair(new AsnNameValuePair(ENTERPRISE_OID_KEY, new AsnOID(this.enterpriseOID)));
        // Add system up time.
        this.trapPdu.addNameValuePair(new AsnNameValuePair(SYSTEM_UPTIME_KEY, new AsnInteger(this.sysUpTime)));
        this.snmpMessage.setPdu(trapPdu);
        this.snmpMessage.setSnmpVersion(2);
        //fire it off
        final SnmpClient client = new SnmpClient();
        try {
            client.sendSnmpMessage(this.managementHost, this.managementHostTrapListenPort, snmpMessage);
        } catch (Exception ex) {
            LogLog.error("There was an unexpected error while sending the trap.", ex);
        }
    }
}