/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software
 * License version 1.1, a copy of which has been included with this
 * distribution in the LICENSE.APL file.  */
package org.apache.log4j.ext;

import org.apache.log4j.helpers.LogLog;

import java.util.*;
import java.io.*;

/**
 * Title: NetSnmpCommandLineTrapSender<br>
 * <br>
 * Description: This class makes use of the NetSNMP <b>snmptrap</b> utility to
 * send traps that implement the underlying SNMP protocol(s).<br>
 * <br>
 * NOTE: this class is intended primarily as a "proof-of-concept", and to serve
 * as an example of a possible usage of a command line trap sending utility with
 * the appender.  This class is not intended for production-grade environments!<br>
 * <br>
 * You can get a copy of NetSNMP <a href="http://net-snmp.sourceforge.net/">here</a>.<br>
 * <br>
 * You will need a copy of the library (e.g. "snmptrap.exe" and "libsnmp.dll" on Win32
 * platforms, at a minimum) to use this class.<br>
 * <br>
 * This sender launches a separate process, calling the NetSnmp utility "snmptrap" to
 * send a trap.  Because this sender's job is to start an operating system process,
 * it needs to know things that are not readily available to it via the normal avenues
 * open to a Log4J Appender.  In particular, it needs to know the location of
 * the NetSnmp binaries (the file system path), and the path to the MIB files used
 * by NetSnmp.  You communicate this information to the sender using Java System
 * Properties.<br>
 * <br>
 * The path to the NetSnmp binaries is set via the property
 * "log4j.ext.snmpTrapAppender.netSnmp.binPath".   If not set, the sender defaults to a value
 * of "/usr/local/bin/".<br>
 * <br>
 * The path to the NetSnmp MIBs is set via the property
 * "log4j.ext.snmpTrapAppender.netSnmp.mibsPath".   If not set, the sender defaults to a value
 * of "/usr/local/share/mibs".<br>
 * <br>
 * You can also get the sender to emit some diagnostic/debugging information by setting the
 * value of the "log4j.ext.snmpTrapAppender.netSnmp.diagnostic" property to "true".<br>
 * <br>
 * Thus, as an example, the following options to the "java" command line set the binary path,
 * the MIB path and the diagnostic flag for this sender:<br>
 * <br>
 * <code>-Dlog4j.ext.snmpTrapAppender.netSnmp.binPath="D:\\apps\\ucd-snmp-4.2.3-win32\\usr\\bin\\"
 * -Dlog4j.ext.snmpTrapAppender.netSnmp.mibsPath="D:\\apps\\ucd-snmp-4.2.3-win32\\usr\\mibs"
 * -Dlog4j.ext.snmpTrapAppender.netSnmp.diagnostic="true"</code><br>
 * <p>
 * @author Mark Masterson (<a href="mailto:m.masterson@computer.org">m.masterson@computer.org</a>)<br>
 * <a href="http://www.m2technologies.net/">http://www.m2technologies.net/</a><br>
 * @version 1.0.0<br>
 * 2002-11-01<br>
 * changes ---<br>
 *
 * 2002-12-10: mwm : minor tweaks and prettying up of code.<br>
 */
public class NetSnmpCommandLineTrapSender implements SnmpTrapSenderFacade {

    private static final String NET_SNMP_CMD_LINE_TRAP_SENDER_DIAGNOSTIC_PROPERTY = "log4j.ext.snmpTrapAppender.netSnmp.diagnostic";
    private static final String NET_SNMP_BIN_PATH_PROPERTY = "log4j.ext.snmpTrapAppender.netSnmp.binPath";
    private static final String NET_SNMP_MIBS_PATH_PROPERTY = "log4j.ext.snmpTrapAppender.netSnmp.mibsPath";
    private static final String NET_SNMP_BIN_PATH_DEFAULT = "/usr/local/bin/";
    private static final String NET_SNMP_MIBS_PATH_DEFAULT = "/usr/local/share/mibs";
    private String managementHost = "127.0.0.1";
    private String enterpriseOID = "1.3.6.1.2.1.1.2.0";
    private String localIPAddress = "127.0.0.1";
    private int genericTrapType = 0;
    private int specificTrapType = 6;
    private String communityString = "public";
    private long sysUpTime = 0;
    private boolean isInitialized = false;
    private List bindVariables;

    public void initialize(final String managementHostValue,
                           final int managementHostTrapListenPortValue,
                           final String enterpriseOIDValue,
                           final String localIPAddressValue,
                           final int localTrapSendPortValue,
                           final int genericTrapTypeValue,
                           final int specificTrapTypeValue,
                           final String communityStringValue,
                           final long sysUpTimeValue) {
        this.managementHost = managementHostValue;
        this.enterpriseOID = enterpriseOIDValue;
        this.localIPAddress = localIPAddressValue;
        this.genericTrapType = genericTrapTypeValue;
        this.specificTrapType = specificTrapTypeValue;
        this.communityString = communityStringValue;
        this.sysUpTime = sysUpTimeValue;
        this.bindVariables = new ArrayList();
        this.isInitialized = true;
    }

    public void addTrapMessageVariable(final String applicationTrapOIDValue,
                                       final String value) {
        //check pre-condition
        if (!this.isInitialized) {
            LogLog.error("The initialize() method must be called before calling addTrapMessageVariable()");
            return;
        }
        this.bindVariables.add(new BindVariable(applicationTrapOIDValue, value));
    }

    public void sendTrap() {
        final String binPath = System.getProperty(NET_SNMP_BIN_PATH_PROPERTY, NET_SNMP_BIN_PATH_DEFAULT);
        final String mibsPath = System.getProperty(NET_SNMP_MIBS_PATH_PROPERTY, NET_SNMP_MIBS_PATH_DEFAULT);
        final boolean diagnosticFlag = Boolean.getBoolean(NET_SNMP_CMD_LINE_TRAP_SENDER_DIAGNOSTIC_PROPERTY);
        if (diagnosticFlag) {
            LogLog.error(binPath);
            LogLog.error(mibsPath);
        }
        final int index = 0;
        final String[] paramsX = this.sizeParameterArray();
        try {
            paramsX[index] = binPath + "snmptrap";
            this.fillParamsArray(paramsX, index, mibsPath);
            if (diagnosticFlag) LogLog.error("Command line array contains: " + Arrays.asList(paramsX));
            final Process einProcess = Runtime.getRuntime().exec(paramsX, null, new File(binPath));
            this.handleProcessStream(new BufferedReader(new InputStreamReader(einProcess.getInputStream())), diagnosticFlag);
            this.handleProcessStream(new BufferedReader(new InputStreamReader(einProcess.getErrorStream())), diagnosticFlag);
            final int rc = einProcess.waitFor();
            if (0 != rc) {
                LogLog.error("Error executing snmptrap!");
            }
        } catch (IOException e) {
            LogLog.error("Error executing snmptrap!", e);
        } catch (InterruptedException e) {
            LogLog.error("Error executing snmptrap!", e);
        }
    }

    private String[] sizeParameterArray() {
        final int mandatoryParameterCount = 9;
        final int variableParameterCount = 3 * this.bindVariables.size();
        return new String[mandatoryParameterCount + variableParameterCount];
    }

    private static void handleProcessStream(final BufferedReader bufferedInputStreamReader, final boolean diagnosticFlag)
            throws IOException, InterruptedException {
        for (int i = 0; i < 99; i++) {
            if (!bufferedInputStreamReader.ready())
                Thread.sleep(100);
            else
                break;
        }
        while (bufferedInputStreamReader.ready()) {
            if (diagnosticFlag) {
                LogLog.error(bufferedInputStreamReader.readLine());
            } else {
                bufferedInputStreamReader.readLine();
            }
        }
        bufferedInputStreamReader.close();
    }

    private void fillParamsArray(final String[] paramsX, int index, final String mibsPath) {
        paramsX[++index] = "-M " + mibsPath;
        paramsX[++index] = this.managementHost;
        paramsX[++index] = this.communityString;
        paramsX[++index] = "." + this.enterpriseOID;
        paramsX[++index] = this.localIPAddress;
        paramsX[++index] = Integer.toString(this.genericTrapType);
        paramsX[++index] = Integer.toString(this.specificTrapType);
        paramsX[++index] = Long.toString(this.sysUpTime);
        for (Iterator varsIt = this.bindVariables.iterator(); varsIt.hasNext();) {
            final BindVariable tmpVar = (BindVariable) varsIt.next();
            paramsX[++index] = "." + tmpVar.getOid();
            paramsX[++index] = "s";
            paramsX[++index] = "\"" + tmpVar.getValue() + "\"";
        }
    }

    private static class BindVariable {
        private final String oid;
        private final String value;

        public BindVariable(final String oidValue, final String variableValue) {
            this.oid = oidValue;
            this.value = variableValue;
        }

        public String getOid() {
            return this.oid;
        }

        public String getValue() {
            return this.value;
        }
    }
}
